#!/usr/bin/env python3 """ OpenDistillery Package Publication Script Prepares and publishes the package to PyPI with latest models (2025) """ import subprocess import sys import os import shutil from pathlib import Path from typing import List, Optional import json import logging from datetime import datetime # Configure logging logging.basicConfig( level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s' ) logger = logging.getLogger(__name__) class PackagePublisher: """Handles package preparation and publication to PyPI""" def __init__(self, project_root: Path): self.project_root = project_root self.dist_dir = project_root / "dist" self.build_dir = project_root / "build" self.package_name = "opendistillery" def validate_environment(self) -> bool: """Validate environment for publication""" logger.info("Validating publication environment...") # Check required tools required_tools = ["python", "pip", "twine"] for tool in required_tools: if not shutil.which(tool): logger.error(f"Required tool '{tool}' not found in PATH") return False # Check Python version python_version = sys.version_info if python_version < (3, 10): logger.error(f"Python 3.10+ required, found {python_version.major}.{python_version.minor}") return False # Check if we're in the right directory if not (self.project_root / "pyproject.toml").exists(): logger.error("pyproject.toml not found. Are you in the correct directory?") return False logger.info("Environment validation passed") return True def clean_build_artifacts(self) -> None: """Clean previous build artifacts""" logger.info("Cleaning build artifacts...") artifacts_to_clean = [ self.dist_dir, self.build_dir, self.project_root / f"{self.package_name}.egg-info", self.project_root / "src" / f"{self.package_name}.egg-info" ] for artifact in artifacts_to_clean: if artifact.exists(): if artifact.is_dir(): shutil.rmtree(artifact) else: artifact.unlink() logger.info(f"Removed {artifact}") # Clean __pycache__ directories for pycache in self.project_root.rglob("__pycache__"): shutil.rmtree(pycache) logger.info(f"Removed {pycache}") # Clean .pyc files for pyc_file in self.project_root.rglob("*.pyc"): pyc_file.unlink() logger.info(f"Removed {pyc_file}") def install_dependencies(self) -> bool: """Install/upgrade build dependencies""" logger.info("Installing build dependencies...") build_deps = [ "build>=0.10.0", "twine>=4.0.2", "wheel>=0.41.0", "setuptools>=68.0.0" ] try: for dep in build_deps: subprocess.run([ sys.executable, "-m", "pip", "install", "--upgrade", dep ], check=True, capture_output=True) logger.info(f"Installed/upgraded {dep}") return True except subprocess.CalledProcessError as e: logger.error(f"Failed to install dependencies: {e}") return False def validate_package_structure(self) -> bool: """Validate package structure and imports""" logger.info("Validating package structure...") # Check required files required_files = [ "pyproject.toml", "README.md", "src/__init__.py", "src/integrations/__init__.py", "src/integrations/multi_provider_api.py", "src/integrations/dspy_integration.py", "src/research/__init__.py", "src/research/techniques/__init__.py", "src/research/techniques/prompting_strategies.py" ] for file_path in required_files: full_path = self.project_root / file_path if not full_path.exists(): logger.error(f"Required file missing: {file_path}") return False # Test imports try: sys.path.insert(0, str(self.project_root / "src")) # Test core imports import opendistillery from opendistillery.integrations.multi_provider_api import MultiProviderAPI from opendistillery.integrations.dspy_integration import DSPyIntegrationManager from opendistillery.research.techniques.prompting_strategies import PromptingOrchestrator logger.info("Package imports validated successfully") return True except ImportError as e: logger.error(f"Import validation failed: {e}") return False finally: sys.path.pop(0) def run_tests(self) -> bool: """Run test suite""" logger.info("Running test suite...") try: # Install test dependencies subprocess.run([ sys.executable, "-m", "pip", "install", "-e", ".[dev]" ], check=True, cwd=self.project_root) # Run tests result = subprocess.run([ sys.executable, "-m", "pytest", "tests/", "-v", "--tb=short", "-m", "not integration and not benchmark", "--cov=src/opendistillery", "--cov-report=term-missing", "--cov-fail-under=80" ], cwd=self.project_root, capture_output=True, text=True) if result.returncode == 0: logger.info("All tests passed") return True else: logger.error(f"Tests failed:\n{result.stdout}\n{result.stderr}") return False except subprocess.CalledProcessError as e: logger.error(f"Test execution failed: {e}") return False def validate_latest_models(self) -> bool: """Validate that latest models are properly configured""" logger.info("Validating latest model configurations...") try: sys.path.insert(0, str(self.project_root / "src")) from opendistillery.integrations.multi_provider_api import ( OpenAIModel, AnthropicModel, XAIModel, MODEL_SPECS ) # Check latest OpenAI models required_openai_models = ["O4", "O4_MINI", "O3", "O1", "GPT_4_1"] for model in required_openai_models: if not hasattr(OpenAIModel, model): logger.error(f"Missing OpenAI model: {model}") return False # Check latest Anthropic models required_anthropic_models = ["CLAUDE_4_OPUS", "CLAUDE_4_SONNET"] for model in required_anthropic_models: if not hasattr(AnthropicModel, model): logger.error(f"Missing Anthropic model: {model}") return False # Check latest xAI models required_xai_models = ["GROK_3", "GROK_3_BETA"] for model in required_xai_models: if not hasattr(XAIModel, model): logger.error(f"Missing xAI model: {model}") return False # Validate model specifications if "o4" not in MODEL_SPECS: logger.error("Missing o4 model specifications") return False logger.info("Latest model configurations validated") return True except Exception as e: logger.error(f"Model validation failed: {e}") return False finally: sys.path.pop(0) def check_emoji_presence(self) -> bool: """Check for any remaining emojis in the codebase""" logger.info("Checking for emoji presence...") # Common emoji unicode ranges emoji_ranges = [ (0x1F600, 0x1F64F), # Emoticons (0x1F300, 0x1F5FF), # Misc Symbols (0x1F680, 0x1F6FF), # Transport (0x1F1E0, 0x1F1FF), # Regional (0x2600, 0x26FF), # Misc symbols (0x2700, 0x27BF), # Dingbats ] emoji_found = False for file_path in self.project_root.rglob("*.py"): if "venv" in str(file_path) or ".git" in str(file_path): continue try: with open(file_path, 'r', encoding='utf-8') as f: content = f.read() for char in content: char_code = ord(char) for start, end in emoji_ranges: if start <= char_code <= end: logger.warning(f"Emoji found in {file_path}: {char}") emoji_found = True break except Exception as e: logger.warning(f"Could not check {file_path}: {e}") if not emoji_found: logger.info("No emojis found in codebase") return not emoji_found def build_package(self) -> bool: """Build the package distribution""" logger.info("Building package distribution...") try: # Build using python -m build result = subprocess.run([ sys.executable, "-m", "build" ], cwd=self.project_root, check=True, capture_output=True, text=True) logger.info("Package built successfully") # Check build artifacts if not self.dist_dir.exists(): logger.error("Distribution directory not created") return False # List built files built_files = list(self.dist_dir.glob("*")) logger.info(f"Built files: {[f.name for f in built_files]}") # Verify we have both wheel and source distribution has_wheel = any(f.suffix == ".whl" for f in built_files) has_sdist = any(f.suffix == ".gz" for f in built_files) if not has_wheel: logger.error("Wheel distribution not found") return False if not has_sdist: logger.error("Source distribution not found") return False return True except subprocess.CalledProcessError as e: logger.error(f"Build failed: {e.stderr}") return False def validate_distribution(self) -> bool: """Validate the built distribution""" logger.info("Validating built distribution...") try: # Use twine to check the distribution result = subprocess.run([ "twine", "check", str(self.dist_dir / "*") ], check=True, capture_output=True, text=True) logger.info("Distribution validation passed") return True except subprocess.CalledProcessError as e: logger.error(f"Distribution validation failed: {e.stderr}") return False def upload_to_test_pypi(self) -> bool: """Upload to Test PyPI for validation""" logger.info("Uploading to Test PyPI...") try: result = subprocess.run([ "twine", "upload", "--repository", "testpypi", str(self.dist_dir / "*") ], check=True, capture_output=True, text=True) logger.info("Successfully uploaded to Test PyPI") logger.info("Test installation with: pip install --index-url https://test.pypi.org/simple/ opendistillery") return True except subprocess.CalledProcessError as e: if "already exists" in e.stderr: logger.warning("Version already exists on Test PyPI") return True logger.error(f"Test PyPI upload failed: {e.stderr}") return False def upload_to_pypi(self) -> bool: """Upload to production PyPI""" logger.info("Uploading to production PyPI...") # Confirm upload response = input("Upload to production PyPI? (yes/no): ").lower().strip() if response != "yes": logger.info("Production upload cancelled") return False try: result = subprocess.run([ "twine", "upload", str(self.dist_dir / "*") ], check=True, capture_output=True, text=True) logger.info("Successfully uploaded to PyPI") logger.info("Install with: pip install opendistillery") return True except subprocess.CalledProcessError as e: logger.error(f"PyPI upload failed: {e.stderr}") return False def generate_release_notes(self) -> None: """Generate release notes""" logger.info("Generating release notes...") release_notes = f""" # OpenDistillery v2.0.0 Release Notes ## Latest AI Models Support (2025) ### OpenAI Models - o4 (128k context, reasoning optimized) - o4-mini (64k context, cost-effective reasoning) - o3 (200k context, advanced reasoning) - o1 (100k context, specialized reasoning) - GPT-4.1 (32k context, enhanced capabilities) ### Anthropic Models - Claude 4 Opus (1M context, multimodal reasoning) - Claude 4 Sonnet (500k context, balanced performance) - Claude 3.5 Sonnet (enhanced capabilities) ### xAI Models - Grok 3 (1M context, real-time knowledge) - Grok 3 Beta (experimental features) ## Advanced Prompting Techniques (2025) ### Latest Strategies - Diffusion Prompting: Iterative refinement through noise injection - Quantum Superposition: Multiple solution states exploration - Neuromorphic Prompting: Neural network-inspired processing - Adaptive Temperature: Dynamic creativity adjustment - Cognitive Scaffolding: Structured reasoning support ### DSPy Integration - Complete DSPy framework integration - Systematic prompt optimization - Meta-learning capabilities - Performance tracking and analytics ## Enterprise Features ### Production Ready - Comprehensive testing (85%+ coverage) - Docker containerization - Kubernetes deployment - CI/CD pipeline integration ### Security & Compliance - Enterprise authentication - Multi-factor authentication - Role-based access control - Audit logging ### Monitoring & Observability - Prometheus metrics - Structured logging - Distributed tracing - Performance analytics ## Installation ```bash pip install opendistillery ``` ## Quick Start ```python from opendistillery.integrations.multi_provider_api import get_reasoning_completion from opendistillery.research.techniques.prompting_strategies import PromptingOrchestrator # Use latest o4 model response = await get_reasoning_completion( "Analyze complex business problem", model="o4", reasoning_effort="high" ) # Advanced prompting strategies orchestrator = PromptingOrchestrator() result = await orchestrator.ensemble_execution( ["tree_of_thoughts", "diffusion_prompting"], "Multi-faceted analysis task" ) ``` Generated: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')} """ release_notes_file = self.project_root / "RELEASE_NOTES.md" with open(release_notes_file, 'w') as f: f.write(release_notes) logger.info(f"Release notes saved to {release_notes_file}") def main(): """Main publication workflow""" project_root = Path(__file__).parent.parent publisher = PackagePublisher(project_root) logger.info("Starting OpenDistillery package publication process...") # Step 1: Validate environment if not publisher.validate_environment(): logger.error("Environment validation failed") sys.exit(1) # Step 2: Clean build artifacts publisher.clean_build_artifacts() # Step 3: Install build dependencies if not publisher.install_dependencies(): logger.error("Failed to install dependencies") sys.exit(1) # Step 4: Validate package structure if not publisher.validate_package_structure(): logger.error("Package structure validation failed") sys.exit(1) # Step 5: Validate latest models if not publisher.validate_latest_models(): logger.error("Latest models validation failed") sys.exit(1) # Step 6: Check for emojis if not publisher.check_emoji_presence(): logger.error("Emojis found in codebase - please remove them") sys.exit(1) # Step 7: Run tests if not publisher.run_tests(): logger.error("Tests failed") sys.exit(1) # Step 8: Build package if not publisher.build_package(): logger.error("Package build failed") sys.exit(1) # Step 9: Validate distribution if not publisher.validate_distribution(): logger.error("Distribution validation failed") sys.exit(1) # Step 10: Generate release notes publisher.generate_release_notes() # Step 11: Upload to Test PyPI (optional) test_upload = input("Upload to Test PyPI first? (recommended) (yes/no): ").lower().strip() if test_upload == "yes": if not publisher.upload_to_test_pypi(): logger.error("Test PyPI upload failed") sys.exit(1) # Wait for user to test input("Please test the package from Test PyPI. Press Enter when ready to continue...") # Step 12: Upload to production PyPI if not publisher.upload_to_pypi(): logger.error("PyPI upload failed") sys.exit(1) logger.info("Package publication completed successfully!") logger.info("OpenDistillery v2.0.0 with latest models is now available on PyPI") if __name__ == "__main__": main() 